from .utils import getEndpointUrl,get_async_response
from abstract_apis import getRequest,requests,asyncPostRpcRequest,asyncPostRequest,asyncGetRequest,get_headers
import json
async def asyncMakeLimitedDbCall(method=None, params=[]):
    response = getRequest(url=getEndpointUrl("dbSearch"), data={"method":method, "params":params},headers=get_headers())
    print(type(response))
    print(response)
    if response != None:
      print(f'search of {method}  successful')
      return response
    urls = await async_get_rate_limit_url(method)
    response = await asyncPostRpcRequest(
        url=urls.get('url'), method=method, params=params, status_code=True, response_result='result'
    )
    
    if response[1] == 429:
        response = await asyncPostRpcRequest(
            url=urls.get('url2'), method=method, params=params, response_result='result', status_code=True
        )
    response = response[0]
    await async_log_response(method,response)
    await asyncPostRequest(url=getEndpointUrl("dbInsert"), data={"method":method, "params":params,"result":response}, status_code=True)
    return response

def makeLimitedDbCall(method=None, params=[]):
    return get_async_response(asyncMakeLimitedDbCall, method, params)

async def asyncMakeLimitedCall(method=None, params=[]):
    urls = await async_get_rate_limit_url(method)
    response = await asyncPostRpcRequest(
        url=urls.get('url'), method=method, params=params, status_code=True, response_result='result'
    )
    
    if response[1] == 429:
        response = await asyncPostRpcRequest(
            url=urls.get('url2'), method=method, params=params, response_result='result', status_code=True
        )
    
    await async_log_response(method, response[0])
    return response[0]

def makeLimitedCall(method=None, params=[]):
    return get_async_response(asyncMakeLimitedCall, method, params)

async def async_get_rate_limit_url(method='default_method'):
    return await asyncGetRequest(url=getEndpointUrl("rate_limit"),data={"method":str(method)})

def get_rate_limit_url(method_name, *args, **kwargs):
    return get_async_response(async_get_rate_limit_url, method_name, *args, **kwargs)

async def async_log_response(method='default_method', response_data={}):
    return await asyncPostRequest(url=getEndpointUrl("log_response"),data={"method":str(method),"response_data":response_data})

def log_response(method_name, response_data, *args, **kwargs):
    return get_async_response(async_log_response, method_name, response_data, *args, **kwargs)



from ..utils import getEndpointUrl
from .abstract_async_call import *
from abstract_utilities import make_list
from abstract_apis import asyncPostRequest,asyncGetRequest
async def asyncPostFlaskRequest(endpoint,**kwargs):
  url = getEndpointUrl(endpoint)
  return await asyncPostRequest(url,kwargs)

async def asyncGetFlaskRequest(endpoint,**kwargs):
  url = getEndpointUrl(endpoint)
  return await asyncGetRequest(url,kwargs)

async def asyncViewTable(table_name, column_name=None, start=None, end=None, filters=None, search_string=None,deep_search=False,latest=None,**kwargs):
  return await asyncPostFlaskRequest('view_table',table_name=table_name,column_name=column_name, start=start, end=end, filters=filters, search_string=search_string,deep_search=deep_search,latest=latest,**kwargs)

async def asyncGetLpKeys(table_name=None, column_name=None, start=None, end=None, filters=None, search_string=None,deep_search=False,latest=None,**kwargs):
  response = await asyncViewTable(table_name=table_name or 'key_info',column_name=column_name, start=start, end=end, filters=filters, search_string=search_string,deep_search=deep_search,latest=latest,**kwargs)
  signature= response[0].get('signature')
  return await asyncGetCallRequest('getLpKeys',signature)

async def asyncList_tables(with_data=True):
  url = getEndpointUrl(endpoint)
  response = await asyncGetRequest(url, data={"with_data":with_data})
  return response

async def asyncList_columns(table_name=None):
  tables = {}
  url = getEndpointUrl(endpoint)
  table_names = make_list(table_name or list_tables(with_data=True))
  for tableName in table_names:
    tables[tableName]= await asyncGetRequest(url, data={"table_name":tableName})
  return tables

async def asyncCallRequest(endpoint,*args,**kwargs):
  url = getEndpointUrl(endpoint)
  return await asyncPostRequest(url,kwargs)

def solcatcherPostRequest(endpoint,**kwargs):
    return asyncio.run(asyncCallRequest(endpoint,**kwargs))
async def get_async_request_data(method, params):
  rpc_url = await async_get_rate_limit_url(method) or {}
  url = rpc_url.get('url')
  response = await asyncPostRpcRequest(url=url ,status_code=True, method=method, params=params)
  status_code = response.get("status_code")
  if status_code == 429:
    url = rpc_url.get('url2')
    response = await asyncPostRpcRequest(url=url ,status_code=True, method=method, params=params)
  response = await async_get_response(response)
  if url != rpc_url.get('url2'):
    await async_log_response(method,response)
  return response
def get_async_response(method, params):
    return asyncio.run(get_async_request_data(method, params))



import asyncio
from abstract_apis import get_url,make_endpoint,get_headers
def getSolcatcherUrl():
  return 'https://solcatcher.io'
def getEndpointUrl(endpoint=None,url=None):
  url = url or getSolcatcherUrl()
  endpoint = make_endpoint(endpoint or '/')
  return get_url(url,endpoint)
def updateData(data,**kwargs):
  data.update(kwargs)
  return data
def getCallArgs(endpoint):
  return {'getMetaData': ['signature'], 'getPoolData': ['signature'], 'getTransactionData': ['signature'], 'getPoolInfo': ['signature'], 'getMarketInfo': ['signature'], 'getKeyInfo': ['signature'], 'getLpKeys': ['signature'], 'process': ['signature']}.get(get_endpoint(endpoint))
def ifListGetSection(listObj,section=0):
    if isinstance(listObj,list):
        if len(listObj)>section:
            return listObj[section]
    return listObj
def get_async_response(async_function, *args, **kwargs):
    try:
        loop = asyncio.get_event_loop()
    except RuntimeError:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

    if not loop.is_running():
        # Only run the loop if it's not already running
        return loop.run_until_complete(async_function(*args, **kwargs))
    else:
        # If the loop is already running, directly run the coroutine
        return asyncio.ensure_future(async_function(*args, **kwargs))



from ..utils import get_async_response,getEndpointUrl,get_headers
from ..async_utils import asyncPostRequest
import json,requests

async def get_solcatcher_endpoint(endpoint,*args,**kwargs):
    return requests.post(url=getEndpointUrl(endpoint),data=json.dumps(kwargs),headers=get_headers())
  
def getTxnTypeFromMint(address):
    return get_async_response(get_solcatcher_endpoint,endpoint="getTxnTypeFromMint",address=address)

def sendTransaction(txn, skipPreflight=None, preflightCommitment=None):
  return get_async_response(get_solcatcher_endpoint,endpoint="sendTransaction",txn=txn, skipPreflight=skipPreflight, preflightCommitment=preflightCommitment)


def getGenesisSignature(address,before=None,limit=None):
  return get_async_response(get_solcatcher_endpoint,endpoint="getGenesisSignature",address=address,before=before,limit=limit)

def getSignaturesForAddress(address, limit=10, before=None, after=None, finalized=None,encoding=None,commitment=None,errorProof=False):
  return get_async_response(get_solcatcher_endpoint,endpoint="getSignaturesForAddress",address=address, limit=limit, before=before, after=after, finalized=finalized,encoding=encoding,commitment=commitment,errorProof=errorProof)

def getTokenAccountBalance(account,mint=None,commitment=None):
  return get_async_response(get_solcatcher_endpoint,endpoint="getTokenAccountBalance",account=account,mint=mint,commitment=commitment)

def getTokenAccountsByOwner(account,mint=None,encoding=None):
  return get_async_response(get_solcatcher_endpoint,endpoint="getTokenAccountsByOwner",account=account,mint=mint,encoding=encoding)

def getMetaData(address):
  return get_async_response(get_solcatcher_endpoint,endpoint="getMetaData",address=address)

def getTransaction(signature,maxSupportedTransactionVersion=None):
  return get_async_response(get_solcatcher_endpoint,endpoint="getTransaction",signature=signature,maxSupportedTransactionVersion=maxSupportedTransactionVersion)

def getLatestBlockHash(commitment=None):
    return get_async_response(get_solcatcher_endpoint,endpoint="getLatestBlockhash",commitment=commitment)

def getAccountInfo(account,encoding=None,commitment=None):
  return get_async_response(get_solcatcher_endpoint,endpoint="getAccountInfo",account=account,encoding=encoding,commitment=commitment)

def getBlock(slot,encoding=None,maxSupportedTransactionVersion=None,transactionDetails= None,rewards=None):
    return get_async_response(get_solcatcher_endpoint,endpoint="getBlock",slot=slot,encoding=encoding,maxSupportedTransactionVersion=maxSupportedTransactionVersion,transactionDetails=transactionDetails,rewards=rewards)


